unit FileIOUnit;

interface

uses
	SysUtils, Classes;

type
	TIdent = ShortInt;

const
	NoIdent = Low (TIdent);
	FileIdent = 'Board Reader File v1';

type
	TFileReader = class(TObject)
	private
		FStream: TStream;
		FReadAheadIdent: TIdent;
		FNonExistingBlockIndent: Integer;
	public
		constructor Create(AStream: TStream);
		function ReadIdent(Requested: TIdent = NoIdent): TIdent;
		procedure ReadIndent;
		procedure ReadUnindent;
		function ReadString(const Std: string = ''): string;
		function ReadInteger(Std: Integer = 0): Integer;
		function ReadBoolean(Std: Boolean = False): Boolean;
		function ReadDateTime(Std: TDateTime = 0): TDateTime;
		property ReadAheadIdent: TIdent read FReadAheadIdent write FReadAheadIdent;
		property Stream: TStream read FStream;
	end;

	TFileWriter = class(TObject)
	private
		FStream: TStream;
	public
		constructor Create(AStream: TStream);
		procedure WriteIdent(Ident: ShortInt);
		procedure WriteIndent;
		procedure WriteUnindent;
		procedure WriteString(const S: string);
		procedure WriteInteger(Value: Integer);
		procedure WriteBoolean(Value: Boolean);
		procedure WriteDateTime(Value: TDateTime);
		property Stream: TStream read FStream;
	end;

	EWrongIdent = class(Exception);
	ENoZeroByte = class(Exception);

implementation

{ TFileReader }

constructor TFileReader.Create(AStream: TStream);
begin
	inherited Create;
	FStream := AStream;
	FReadAheadIdent := NoIdent;
end;

function TFileReader.ReadIdent(Requested: TIdent): TIdent;
begin
	if ReadAheadIdent = NoIdent then try
		Stream.ReadBuffer (Result, 1)
	except
		Result := 0;
	end else begin
		Result := ReadAheadIdent;
		ReadAheadIdent := NoIdent;
	end;
	if (Requested <> NoIdent) and (Result <> Requested) then begin
		ReadAheadIdent := Result;
		raise EWrongIdent.CreateFmt ('Wrong ident number (found: %d; requested: %d)', [Integer (Result), Integer (Requested)]);
	end;
end;

procedure TFileReader.ReadIndent;
begin
	try
		ReadIdent (-1);
	except
		if ReadAheadIdent = 0 then
			Inc (FNonExistingBlockIndent)
		else
			raise;
	end;
end;

procedure TFileReader.ReadUnindent;
begin
	if FNonExistingBlockIndent > 0 then
		Dec (FNonExistingBlockIndent)
	else
		ReadIdent (0);
end;

function TFileReader.ReadString(const Std: string): string;
var
	Len: Cardinal;
	EndByte: Byte;
begin
	Result := Std;
	try
		ReadIdent (1);
	except
		if ReadAheadIdent = 0 then
			Exit
		else
			raise;
	end;
	Stream.ReadBuffer (Len, SizeOf (Len));
	SetLength (Result, Len);
	if Len > 0 then
		Stream.ReadBuffer (Result [1], Len);
	Stream.ReadBuffer (EndByte, 1);
	if EndByte <> 0 then
		raise ENoZeroByte.Create ('Unterminated string');
end;

function TFileReader.ReadInteger(Std: Integer): Integer;
begin
	Result := Std;
	try
		ReadIdent (2);
	except
		if ReadAheadIdent = 0 then
			Exit
		else
			raise;
	end;
	Stream.ReadBuffer (Result, SizeOf (Result));
end;

function TFileReader.ReadBoolean(Std: Boolean): Boolean;
var
	B: Byte;
begin
	try
		ReadIdent (3);
	except
		if ReadAheadIdent = 0 then begin
			Result := Std;
			Exit;
		end else
			raise;
	end;
	Stream.ReadBuffer (B, 1);
	Result := B <> 0;
end;

function TFileReader.ReadDateTime(Std: TDateTime): TDateTime;
begin
	try
		ReadIdent (4);
	except
		if ReadAheadIdent = 0 then begin
			if Std = 0 then
				Std := Now;
			Result := Std;
			Exit;
		end else
			raise;
	end;
	Stream.ReadBuffer (Result, SizeOf (Result));
end;

{ TFileWriter }

constructor TFileWriter.Create(AStream: TStream);
begin
	inherited Create;
	FStream := AStream;
end;

procedure TFileWriter.WriteIdent(Ident: ShortInt);
begin
	Stream.WriteBuffer (Ident, 1);
end;

procedure TFileWriter.WriteIndent;
begin
	WriteIdent (-1);
end;

procedure TFileWriter.WriteUnindent;
begin
	WriteIdent (0);
end;

procedure TFileWriter.WriteString(const S: string);
var
	Len: Cardinal;
begin
	WriteIdent (1);
	Len := Length (S);
	Stream.WriteBuffer (Len, SizeOf (Len));
	Stream.WriteBuffer (PChar(S)^, Length (S) + 1);
end;

procedure TFileWriter.WriteInteger(Value: Integer);
begin
	WriteIdent (2);
	Stream.WriteBuffer (Value, SizeOf (Value));
end;

procedure TFileWriter.WriteBoolean(Value: Boolean);
var
	B: Byte;
begin
	WriteIdent (3);
	if Value then
		B := 1
	else
		B := 0;
	Stream.WriteBuffer (B, 1);
end;

procedure TFileWriter.WriteDateTime(Value: TDateTime);
begin
	WriteIdent (4);
	Stream.WriteBuffer (Value, SizeOf (Value));
end;

end.
