unit MasterUnit;

interface

uses
	SysUtils, Classes;

type
	TRequestAction = (raNone, raGlobalLogin, raLocalLogin, raRequestUpdates, raRequestAll, raMarkForumRead, raRequestTopic, raPostTopic, raPostReply, raEditPost, raDeleteTopic, raDeletePost, raReloadPost);
	TConnectionStatus = (csReady, csConnecting, csStarting, csDownloading);
	TPostExtra = (peEZCodes, peEmoticons, peAutoLinks, peHTML);
	TPostExtras = set of TPostExtra;

const
	StdShortDateFormat = 'yyyy-mm-dd';
	StdShortTimeFormat = 'hh:nn';

function StringReplace(const S, OldPattern, NewPattern: string; Flags: TReplaceFlags): string;

function ExtractHost(const URL: string): string;
function ExtractRelPath(const URL: string): string;
function ExtractURLID(const URL: string): string;
function ExtractURLParams(const URL: string): string;

function HTMLize(const Input: string): string;
function DeHTMLize(Input: string): string;
function URLEncode(const Input: string): string;

function DeleteToFirst(SubStr: string; var Str: string; out Part: string; MaxLen: Integer = -1): Boolean;
function DeleteToAfterFirst(const SubStr: string; var Str: string; out Part: string; MaxLen: Integer = -1): Boolean;
function DeleteToAfterFirstAndAdd(const SubStr: string; var Str: string; out Part: string; MaxLen: Integer = -1): Boolean;
procedure DeleteWhiteSpace(var Str: string);
procedure TrimWhiteSpace(var Str: string);
procedure TrimHTMLWhiteSpace(var Str: string);
procedure TrimHTML(var Str: string; Pre: Boolean = False);

const
	ButtonMargin = 4;

var
	ProgStartTime: TDateTime;

implementation

function StringReplace(const S, OldPattern, NewPattern: string; Flags: TReplaceFlags): string;
var
	P1, P2: Integer;
begin
	if Flags = [rfReplaceAll] then begin
		Result := S;
		P1 := 0;
		repeat
			P2 := Pos (OldPattern, Copy (Result, P1 + 1, Length (Result)));
			if P2 > 0 then begin
				Inc (P1, P2);
				Delete (Result, P1, Length (OldPattern));
				Insert (NewPattern, Result, P1);
			end;
		until P2 <= 0;
	end else
		Result := SysUtils.StringReplace (S, OldPattern, NewPattern, Flags);
end;

function ExtractHost(const URL: string): string;
begin
	Result := URL;
	if Copy (Result, 1, Length ('http://')) = 'http://' then
		Delete (Result, 1, Length ('http://'));
	if Pos ('/', Result) > 0 then
		Delete (Result, Pos ('/', Result), Length (Result));
end;

function ExtractRelPath(const URL: string): string;
begin
	Result := URL;
	if Copy (Result, 1, Length ('http://')) = 'http://' then
		Delete (Result, 1, Length ('http://'));
	if Pos ('/', Result) > 0 then
		Delete (Result, 1, Pos ('/', Result) - 1)
	else
		Result := '/';
end;

function ExtractURLID(const URL: string): string;
begin
	Result := ExtractRelPath (URL);
	if Length (Result) > 0 then begin
		while Pos ('/', Result) > 0 do
			Delete (Result, 1, Pos ('/', Result));
		if Pos ('?', Result) > 0 then
			Delete (Result, Pos ('?', Result), Length (Result));
	end;
end;

function ExtractURLParams(const URL: string): string;
begin
	if Pos ('?', URL) > 0 then
		Result := Copy (URL, Pos ('?', URL) + 1, Length (URL))
	else
		Result := '';
end;

function HTMLize(const Input: string): string;
begin
	Result := Input;
	Result := StringReplace (Result, '&', '&amp;', [rfReplaceAll]);
	Result := StringReplace (Result, '<', '&lt;', [rfReplaceAll]);
	Result := StringReplace (Result, '>', '&gt;', [rfReplaceAll]);
	Result := StringReplace (Result, '"', '&quot;', [rfReplaceAll]);
end;

function DeHTMLize(Input: string): string;
begin
	Result := Input;
	Result := StringReplace (Result, '&nbsp;', ' ', [rfReplaceAll]);
	Result := StringReplace (Result, '&lt;', '<', [rfReplaceAll]);
	Result := StringReplace (Result, '&gt;', '>', [rfReplaceAll]);
	Result := StringReplace (Result, '&quot;', '"', [rfReplaceAll]);
	Result := StringReplace (Result, '&amp;', '&', [rfReplaceAll]);
	Result := StringReplace (Result, '&nbsp ', ' ', [rfReplaceAll]);
end;

function URLEncode(const Input: string): string;
var
	I: Integer;
begin
	Result := '';
	for I := 1 to Length (Input) do
		if Input [I] = ' ' then
			Result := Result + '+'
		else if Input [I] in ['A'..'Z', 'a'..'z', '0'..'9', '.'] then
			Result := Result + Input [I]
		else
			Result := Result + '%' + IntToHex (Byte (Input [I]), 2);
end;

function DeleteToFirst(SubStr: string; var Str: string; out Part: string; MaxLen: Integer): Boolean;
var
	I: Integer;
begin
	Part := '';
	Result := False;
	if (Length (SubStr) > 0) and (Length (Str) > 0) then begin
		SubStr := UpperCase (SubStr);
		if (MaxLen < 0) or (MaxLen > Length (Str) - Length (SubStr)) then
			MaxLen := Length (Str) - Length (SubStr);
		for I := 1 to MaxLen + 1 do begin
			if UpCase (Str [I]) = SubStr [1] then begin
				if UpperCase (Copy (Str, I, Length (SubStr))) = SubStr then begin
					Part := Copy (Str, 1, I - 1);
					Delete (Str, 1, I - 1);
					Result := True;
					Break;
				end;
			end;
		end;
	end;
end;

function DeleteToAfterFirst(const SubStr: string; var Str: string; out Part: string; MaxLen: Integer): Boolean;
begin
	Result := DeleteToFirst (SubStr, Str, Part, MaxLen);
	if Result then
		Delete (Str, 1, Length (SubStr));
end;

function DeleteToAfterFirstAndAdd(const SubStr: string; var Str: string; out Part: string; MaxLen: Integer): Boolean;
begin
	Result := DeleteToAfterFirst (SubStr, Str, Part, MaxLen);
	if Result then
		Part := Part + SubStr;
end;

procedure DeleteWhiteSpace(var Str: string);
begin
	while (Length (Str) > 0) and (Str [1] in [' ', #9, #13, #10]) do
		Delete (Str, 1, 1);
end;

procedure TrimWhiteSpace(var Str: string);
begin
	DeleteWhiteSpace (Str);
	while (Length (Str) > 0) and (Str [Length (Str)] in [' ', #9, #13, #10]) do
		Delete (Str, Length (Str), 1);
end;

procedure TrimHTMLWhiteSpace(var Str: string);
var
	Stop: Boolean;
begin
	repeat
		Stop := True;
		TrimWhiteSpace (Str);
		if Length (Str) >= 4 then begin
			if UpperCase (Copy (Str, 1, 4)) = '<BR>' then begin
				Delete (Str, 1, 4);
				Stop := False;
			end else if Copy (Str, 1, 6) = '&nbsp;' then begin
				Delete (Str, 1, 6);
				Stop := False;
			end else if UpperCase (Copy (Str, 1, 3)) = '<P>' then begin
				Delete (Str, 1, 3);
				Stop := False;
			end else if UpperCase (Copy (Str, Length (Str) - 3, 4)) = '<BR>' then begin
				Delete (Str, Length (Str) - 3, 4);
				Stop := False;
			end else if Copy (Str, Length (Str) - 5, 6) = '&nbsp;' then begin
				Delete (Str, Length (Str) - 5, 6);
				Stop := False;
			end else if UpperCase (Copy (Str, Length (Str) - 3, 4)) = '</P>' then begin
				Delete (Str, Length (Str) - 3, 4);
				Stop := False;
			end;
		end;
	until Stop;
end;

procedure TrimHTML(var Str: string; Pre: Boolean = False);
var
	P1,
	P2: Integer;
	S1,
	S2,
	NewStr: string;
begin
	Str := AdjustLineBreaks (StringReplace (Str, #13#13, #13, [rfReplaceAll]), tlbsCRLF);
	NewStr := '';
	while DeleteToAfterFirst ('<', Str, S1) do begin
		DeleteToAfterFirst ('>', Str, S2);
		if not Pre then
			S1 := StringReplace (S1, #13#10, ' ', [rfReplaceAll]);
		NewStr := NewStr + S1;
		if LowerCase (S2) = 'br' then begin
			if Pre then
				NewStr := NewStr + ' '
			else
				NewStr := NewStr + #13#10;
		end else if LowerCase (S2) = 'blockquote' then
			NewStr := NewStr + #13#10#13#10'<< '
		else if LowerCase (S2) = '/blockquote' then
			NewStr := NewStr + ' >>'#13#10#13#10
		else if LowerCase (Copy (S2, 1, Length ('pre'))) = 'pre' then begin
			if not DeleteToAfterFirst ('</pre>', Str, S1) then begin
				S1 := Str;
				Str := '';
			end;
			S1 := StringReplace (S1, '&quot ;', '&quot;', [rfReplaceAll]);
			S1 := StringReplace (S1, '&nbsp;', ' ', [rfReplaceAll]);
			S1 := StringReplace (S1, '&nbsp ', ' ', [rfReplaceAll]);
			S1 := StringReplace (S1, '&nbsp', ' ', [rfReplaceAll]);
			S1 := StringReplace (S1, ' ', '&nbsp;', [rfReplaceAll]);
			TrimHTML (S1, True);
			NewStr := NewStr + #13#10 + S1;
		end else
			NewStr := NewStr + ' ';
	end;
	Str := NewStr + Str;
	TrimHTMLWhiteSpace (Str);
	if Pre then
		Exit;
	while Pos ('  ', Str) > 0 do
		Str := StringReplace (Str, '  ', ' ', [rfReplaceAll]);
	Str := StringReplace (Str, #13#10' ', #13#10, [rfReplaceAll]);
	Str := StringReplace (Str, ' '#13#10, #13#10, [rfReplaceAll]);
	while Pos (#13#10#13#10#13#10, Str) > 0 do
		Str := StringReplace (Str, #13#10#13#10#13#10, #13#10#13#10, [rfReplaceAll]);
	P1 := 0;
	repeat
		P2 := Pos (' ', Copy (Str, P1 + 1, Length (Str)));
		if P2 > 0 then begin
			Inc (P1, P2);
			if P1 + 1 <= Length (Str) then begin
				if Str [P1 + 1] in [',', ':', ';', '!', '?', ')', ']', '}'] then begin
					if (P1 + 2 > Length (Str)) or (not (Str [P1 + 2] in ['-', '(', '[', '{', ')', ']', '}'])) then
						Delete (Str, P1, 1);
				end;
			end;
			if P1 - 1 >= 1 then begin
				if Str [P1 - 1] in ['(', '[', '{'] then
					if (P1 - 2 < 1) or (not (Str [P1 - 2] in ['-', ':', ';'])) then
						Delete (Str, P1, 1);
			end;
		end;
	until P2 <= 0;
	Str := DeHTMLize (Str);
end;

initialization
	ProgStartTime := Now;
end.
