unit TopicContentsUnit;

interface

uses
	MasterUnit, TopicUnit,
	SysUtils, Classes;

type
	TTopicContents = class(TCollection)
	private
		FTopic: TTopic;
		FNewPost: string;
    FNewPostTitle: string;
		function GetContainsDeleted: Boolean;
		procedure SetNewPost(const Value: string);
	public
		constructor Create(ATopic: TTopic);
		destructor Destroy; override;
		procedure Load;
		procedure Save;
		property Topic: TTopic read FTopic;
		property ContainsDeleted: Boolean read GetContainsDeleted;
		property NewPost: string read FNewPost write SetNewPost;
		property NewPostTitle: string read FNewPostTitle write FNewPostTitle;
	end;

	TTopicPost = class(TCollectionItem)
	private
		FList: TTopicContents;
		FTitle: string;
		FText: string;
		FUserID: string;
		FEditorID: string;
		FPostTime: TDateTime;
		FEditTime: TDateTime;
		FLinkURLs: TStrings;
		FPostExtras: TPostExtras;
    FPostIndex: Integer;
    function GetCanDelete: Boolean;
    function GetCanEdit: Boolean;
	public
		constructor Create(Collection: TCollection); override;
		destructor Destroy; override;
		property List: TTopicContents read FList;
		property PostIndex: Integer read FPostIndex write FPostIndex;
		property UserID: string read FUserID write FUserID;
		property PostTime: TDateTime read FPostTime write FPostTime;
		property Title: string read FTitle write FTitle;
		property Text: string read FText write FText;
		property EditorID: string read FEditorID write FEditorID;
		property EditTime: TDateTime read FEditTime write FEditTime;
		property LinkURLs: TStrings read FLinkURLs;
		property CanEdit: Boolean read GetCanEdit;
		property CanDelete: Boolean read GetCanDelete;
		property PostExtras: TPostExtras read FPostExtras write FPostExtras;
	end;

implementation

uses
	FileIOUnit,
	MainUnit;

{ TTopicContents }

constructor TTopicContents.Create(ATopic: TTopic);
begin
	inherited Create (TTopicPost);
	FTopic := ATopic;
end;

destructor TTopicContents.Destroy;
begin
	NewPost := '';
	inherited;
end;

function TTopicContents.GetContainsDeleted: Boolean;
var
	I: Integer;
begin
	Result := False;
	for I := 0 to Count - 1 do
		with TTopicPost (Items [I]) do
			if PostIndex <> Index then begin
				Result := True;
				Break;
			end;
end;

procedure TTopicContents.Load;
var
	Stream: TFileStream;
	S: string;
	EndByte: Byte;
	Post: TTopicPost;
begin
	S := Topic.FileName;
	if FileExists (S) then begin
		Stream := TFileStream.Create (S, fmOpenRead or fmShareDenyWrite);
		try
			SetLength (S, Length (FileIdent));
			try
				Stream.ReadBuffer (S [1], Length (FileIdent));
				if S <> FileIdent then
					raise EWrongIdent.Create ('Unknown file type');
				Stream.ReadBuffer (EndByte, 1);
				if EndByte <> 0 then
					raise ENoZeroByte.Create ('Unterminated string');
			except
				Abort;
			end;
			with TFileReader.Create (Stream) do try
				ReadIndent;
				BeginUpdate;
				try
					Clear;
					while True do begin
						ReadAheadIdent := ReadIdent;
						if ReadAheadIdent = -1 then begin
							Post := TTopicPost (Add);
							with Post do begin
								ReadIndent;
								UserID := ReadString;
								Title := ReadString;
								Text := StringReplace (ReadString, '<br>', '', [rfReplaceAll, rfIgnoreCase]);
								PostTime := ReadDateTime;
								EditorID := ReadString;
								EditTime := ReadDateTime;
								ReadIndent;
								while True do begin
									ReadAheadIdent := ReadIdent;
									if ReadAheadIdent = -1 then begin
										ReadIndent;
										LinkURLs.Add (ReadString);
										ReadUnindent;
									end else
										Break;
								end;
								ReadUnindent;
								PostIndex := ReadInteger (Index);
								PostExtras := TPostExtras (Byte (ReadInteger));
								ReadUnindent;
							end;
						end else
							Break;
					end;
				finally
					EndUpdate;
				end;
				ReadUnindent;
			finally
				Free;
			end;
		finally
			Stream.Free;
		end;
	end;
end;

procedure TTopicContents.Save;
var
	I,
	J: Integer;
	Stream: TFileStream;
begin
	Stream := TFileStream.Create (Topic.FileName, fmCreate or fmShareExclusive);
	try
		Stream.WriteBuffer (PChar(FileIdent)^, Length (FileIdent) + 1);
		with TFileWriter.Create (Stream) do try
			WriteIndent;
			try
				for I := 0 to Count - 1 do
					with TTopicPost (Items [I]) do begin
						WriteIndent;
						try
							WriteString (UserID);
							WriteString (Title);
							WriteString (Text);
							WriteDateTime (PostTime);
							WriteString (EditorID);
							WriteDateTime (EditTime);
							WriteIndent;
							try
								with LinkURLs do
									for J := 0 to Count - 1 do begin
										WriteIndent;
										try
											WriteString (Strings [J]);
										finally
											WriteUnindent;
										end;
									end
							finally
								WriteUnindent;
							end;
							WriteInteger (PostIndex);
							WriteInteger (Byte (PostExtras));
						finally
							WriteUnindent;
						end;
					end;
			finally
				WriteUnindent;
			end;
		finally
			Free;
		end;
	finally
		Stream.Free;
	end;
end;

procedure TTopicContents.SetNewPost(const Value: string);
begin
	if Length (FNewPost) > 0 then
		Dec (MainForm.NewPostCount);
	FNewPost := Value;
	if Length (FNewPost) > 0 then
		Inc (MainForm.NewPostCount);
end;

{ TTopicPost }

constructor TTopicPost.Create(Collection: TCollection);
begin
	inherited;
	FList := TTopicContents (Collection);
	FPostIndex := Index;
	FPostTime := ProgStartTime;
	FEditTime := ProgStartTime;
	FLinkURLs := TStringList.Create;
end;

destructor TTopicPost.Destroy;
begin
	FLinkURLs.Free;
	inherited;
end;

function TTopicPost.GetCanDelete: Boolean;
begin
	with MainForm do
		Result := CanDeleteAll or (CanDeleteOwn and (UserID = OwnID));
end;

function TTopicPost.GetCanEdit: Boolean;
begin
	with MainForm do
		Result := CanEditAll or (CanEditOwn and (UserID = OwnID));
end;

end.
